from dataclasses import dataclass
from getpass import getpass
from functools import reduce
import sys

from wizlib.parser import WizParser
from vernum.error import VerNumError

from vernum.command import VerNumCommand
from vernum.scheme import Scheme


class NextCommand(VerNumCommand):
    """Increment the version"""

    name = 'next'

    @classmethod
    def add_args(cls, parser: WizParser):
        super().add_args(parser)
        parser.add_argument('increment')

    @VerNumCommand.wrap
    def execute(self):
        if not hasattr(self, 'increment'):
            raise VerNumError(f"VerNum 'next' command requires an increment")
        if self.increment not in self.scheme.INCREMENTS:
            raise VerNumError(f"Invalid increment '{self.increment}' " +
                              "for scheme '{self.scheme.name}'")
        current = self.scheme.parse(self.stripped)
        if current:
            next = current.increment(self.increment)
            if not next:
                raise VerNumError(f"Invalid increment '{self.increment} " +
                                  f"for version '{current}'")
            self.status = f"Version number incremented " + \
                f"from {current} to {next}"
            return str(next)
        else:
            raise VerNumError(f"Invalid version string '{self.stripped}' " +
                              f"for scheme '{self.scheme.name}'")

    @property
    def stripped(self):
        return self.input.text.strip()



from vernum.scheme import Scheme


class MajorScheme(Scheme):
    """Just a number"""

    name = 'major'

    FORMAT = r"^v?(\d+)$"
    INCREMENTS = ['major']

    def increment(self, up: str = 'major'):
        major, = self.val
        if up == 'major':
            major += 1
        return MajorScheme(major,)



from vernum.scheme import Scheme


class PatchScheme(Scheme):
    """Three part version numbers always with major, minor, and patch"""

    name = 'patch'

    FORMAT = r"^v?(\d+)\.(\d+)\.(\d+)$"

    INCREMENTS = ['major', 'minor', 'patch']

    # PATTERN = {
    #     'major': (+1, [0], [0]),
    #     'minor': (+0, +1, [0]),
    #     'patch': (+0, +0, +1)
    # }

    @classmethod
    def zero(cls):
        return cls(0, 0, 0)

    def __str__(self):
        if self.val:
            return '.'.join(str(i) for i in self.val)

    def increment(self, up: str):
        major, minor, patch = self.val
        if up == 'patch':
            patch += 1
        elif up == 'minor':
            minor += 1
            patch = 0
        elif up == 'major':
            major += 1
            minor = patch = 0
        return PatchScheme(major, minor, patch)
